<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://www.biuro.lt
 * @since      1.0.0
 *
 * @package    Biuro_Contacts
 * @subpackage Biuro_Contacts/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Biuro_Contacts
 * @subpackage Biuro_Contacts/public
 * @author     Biuro <info@biuro.lt>
 */
class Biuro_Contacts_Public {

    /**
     * The ID of this plfpisugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    const FIELDS = [
        'employees' => ['name', 'phone', 'email'],
        'employers' => ['name', 'phone', 'email'],
        'position' => ['name', 'phone', 'email', 'city', 'city_id', 'message', 'confirm', 'job_id', 'cv'],
        'recommend' => ['name', 'phone', 'email', 'city', 'city_id', 'recommend_name', 'recommend_phone', 'recommend_email', 'recommend_city', 'recommend_city_id'],
    ];

    const DB_NAMES = [
        'employees' => ['name', 'phone', 'email'],
        'employers' => ['name', 'phone', 'email'],
        'position' => ['name', 'phone', 'email', 'city', 'city_id', 'message', 'confirm', 'job_id'],
        'recommend' => ['name', 'phone', 'email', 'city', 'city_id', 'recommend_name', 'recommend_phone', 'recommend_email', 'recommend_city', 'recommend_city_id'],
    ];

    const BIURO_INSIGHTLY =[
        1 => 'Biuro Lithuania',
        2 => 'Biuro Latvia',
        3 => 'Biuro Estonia',
    ];

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of the plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version ) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;

    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Biuro_Contacts_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Biuro_Contacts_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        // wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/biuro-contacts-public.css', array(), $this->version, 'all' );

    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Biuro_Contacts_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Biuro_Contacts_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        // wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/biuro-contacts-public.js', array( 'jquery' ), $this->version, false );

    }

    public static function insert_row_to_db($data, $format, $customName = 'biuro_employees') {
        global $wpdb;

        $table = $wpdb->prefix . $customName;

        $wpdb->insert( $table, $data, $format );

        return $wpdb->insert_id;
    }

    public static function update_row_in_db($data, $insert_id, $format, $customName = 'biuro_employees') {
        global $wpdb;

        $table = $wpdb->prefix . $customName;

        $where = array('id' => $insert_id);

        $where_format = array('%d');

        return $wpdb->update( $table, $data, $where, $format, $where_format );
    }


    public static function delete_sessions( $str ) {
        foreach (static::FIELDS[$str] as $key) {
           unset($_SESSION[$str . '--' . $key . '-message']);
           unset($_SESSION[$str . '--' . $key . '-status']);
           unset($_SESSION[$str . '--' . $key . '-value']);
        }
    }

    public static function validate( $str, $key, $value, $post ) {
        $empty = ['status' => '', 'message' => '', 'translation' => ''];
        $success = ['status' => 'success', 'message' => '', 'translation' => ''];
        $required = ['status' => 'error', 'message' => 'This field is required', 'translation' => __('This field is required', 'biuro-contacts')];

        switch($key) {
        case 'name':
        case 'recommend_name':

            return (!$value) ? $required : $success;

        case 'phone':
            if (!$value && !is_email($post['email'])):
                return [
                    'status' => 'error',
                    'message' => 'Phone or email field is required',
                    'translation' => __('Phone or email field is required', 'biuro-contacts')
                ];
            endif;

            if (!$value):
                return $empty;
            endif;

            if ( strlen($value) < 7 || !preg_match('/^[\d|\s|\+\-]+$/', $value) ):
                return [
                    'status' => 'error',
                    'message' => 'Incorrect phone number',
                    'translation' => __('Incorrect phone number', 'biuro-contacts')
                ];
            endif;

            return $success;

        case 'email':
            if (!$post['phone'] && !$value):
                return [
                    'status' => 'error',
                    'message' => 'Email or phone field is required',
                    'translation' => __('Email or phone field is required', 'biuro-contacts')
                ];
            endif;

            if (!$post['phone'] && !is_email($value)):
                return [
                    'status' => 'error',
                    'message' => 'Email format is incorrect',
                    'translation' => __('Email format is incorrect', 'biuro-contacts')
                ];
            else:

                if (!$value):
                    return $empty;
                endif;

                return $success;
            endif;

        case 'recommend_phone':
            if (!$value && !is_email($post['recommend_email'])):
                return [
                    'status' => 'error',
                    'message' => 'Friend phone or email field is required',
                    'translation' => __('Friend phone or email field is required', 'biuro-contacts')
                ];
            endif;

            if (!$value):
                return $empty;
            endif;

            if ( strlen($value) < 7 || !preg_match('/^[\d|\s|\+\-]+$/', $value) ):
                return [
                    'status' => 'error',
                    'message' => 'Incorrect phone number',
                    'translation' => __('Incorrect phone number', 'biuro-contacts')
                ];
            endif;

            return $success;

        case 'recommend_email':
            if (!$post['recommend_phone'] && !$value):
                return [
                    'status' => 'error',
                    'message' => 'Friend email or phone field is required',
                    'translation' => __('Friend email or phone field is required', 'biuro-contacts')
                ];
            endif;

            if (!$post['recommend_phone'] && !is_email($value)):
                return [
                    'status' => 'error',
                    'message' => 'Email format is incorrect',
                    'translation' => __('Email format is incorrect', 'biuro-contacts')
                ];
            else:

                if (!$value):
                    return $empty;
                endif;

                return $success;
            endif;

        case 'agree':
            if ($value != "1"):
                return [
                    'status' => 'error',
                    'message' => 'You have to agree with conditions',
                    'translation' => __('You have to agree with conditions', 'biuro-contacts')
                ];
            endif;

            return $success;

        case 'cv':
        case 'city':
        case 'city_id':
        case 'recommend_city':
        case 'recommend_city_id':
        case 'message':
        case 'confirm':
        case 'job_id':
        case 'company':
            return $empty;
        }

        return $success;
    }

    public static function getValue( $key, $value ) {

        switch($key) {
        case 'name':
        case 'phone':
        case 'city':
        case 'recommend_name':
        case 'recommend_phone':
        case 'recommend_city':
        case 'company':
            return sanitize_text_field($value);
        case 'email':
        case 'recommend_email':
            return sanitize_email($value);
        case 'message':
            return sanitize_textarea_field($value);
        case 'cv':
            return sanitize_text_field($value);
        case 'agree':
        case 'city_id':
        case 'recommend_city_id':
        case 'job_id':
        case 'confirm':
            return filter_var($value, FILTER_VALIDATE_INT) ? $value : 0;
        }

        return '';
    }

    public static function isCaptchaValid($post, $section) {
        $code = $post['captcha'] ? $post['captcha'] : '';
        $name = $post['name'] ? $post['name'] : '';

        if ($code) {
            $response = file_get_contents("https://www.google.com/recaptcha/api/siteverify?secret=6Lc76qoUAAAAAHfGWVZFEzFeYAVpCkhDSoMYvVUi&response=".$code."&remoteip=".$_SERVER['REMOTE_ADDR']);
            $response = json_decode($response, true);

            if ( !$response['success'] || $response['score'] < 0.3 ) {

                $handle = fopen('recaptcha.log', 'a+');
                $content = date('Y-m-d H:i:s') . ' - ' . $_SERVER['REMOTE_ADDR'] . ' - ' . $_SERVER['SERVER_NAME'] . ' - ' . $_SERVER['HTTP_REFERER'] . ' - name: ' . $name . PHP_EOL;
                $content .= 'Recaptcha: success: ' . $response['success'] . ', score: ' . $response['score'] . ', action: ' . $response['action'] . ',  challenge_ts: ' . $response['challenge_ts'] . ', action: ' . $response['action'] . PHP_EOL;

                if (isset($response['error-codes']) && is_array($response['error-codes']) && !empty($response['error-codes'])):
                    $content .= 'Error codes:';

                    foreach ($response['error-codes'] as &$code) :
                        $content .= ' ' . $code;
                    endforeach;

                    $content .= PHP_EOL;
                endif;

                $content .= PHP_EOL;

                fwrite($handle, $content);
                fclose($handle);

                static::sendBlockedEmail( static::emailBlockedContent($post, true, $response), $section );

                return false;
            }

            return true;
        }

        $handle = fopen('recaptcha.log', 'a+');
        $content = date('Y-m-d H:i:s') . ' - ' . $_SERVER['REMOTE_ADDR'] . ' - ' . $_SERVER['SERVER_NAME'] . ' - ' . $_SERVER['HTTP_REFERER'] . ' - name: ' . $name . PHP_EOL;
        $content .= 'Recaptcha: no recaptcha code' . PHP_EOL . PHP_EOL;
        fwrite($handle, $content);
        fclose($handle);

        static::sendBlockedEmail( static::emailBlockedContent($post, false), $section );

        return false;
    }

    /**
     * Processes shortcode biuro-contacts--employees
     *
     * @param   array   $attr       The attributes from the shortcode
     *
     * @return  mixed   $output     Output of the buffer
     */
    public function employees_form( $attr = array() ) {
        ob_start();

        include_once('partials/biuro-contacts-public--employees.php');

        $output = ob_get_contents();

        ob_end_clean();

        static::delete_sessions('employees');

        return $output;
    } // employees_form()

    function change_upload_dir( $dir ) {
        return array(
            'path'   => $dir['basedir'] . '/additional',
            'url'    => $dir['baseurl'] . '/additional',
            'subdir' => '/additional',
        ) + $dir;
    }

    function change_upload_filename( $filename ) {
        $length = 24;
        $info = pathinfo( $filename );
        $ext  = empty( $info['extension'] ) ? '' : '.' . $info['extension'];
        $name = basename( $filename, $ext );

        return substr(str_shuffle(str_repeat($x='0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ', ceil($length/strlen($x)) )),1,$length) . '-' . $name.$ext;
    }

    public function employees_post() {

        $nonce = $_POST['_wpnonce'];
        $referer = $_POST['_wp_http_referer'];

        if ( !isset( $nonce ) || !wp_verify_nonce($nonce, 'employees_post_nonce' )) {
            wp_redirect( $referer );
            exit;
        }

        if ( $referer && substr($referer, -1) == '?') {
            $referer = substr($referer, 0, -1);
        }

        $str = 'employees';

        if ($_POST['s'] == "1"):
            // static::set_sessions('employees', $_POST);
            $canSubmit = true;

            $data = [
                'created' => current_time('Y-m-d H:i:s')
            ];

            foreach (static::FIELDS[$str] as $key) {
                $value = static::getValue($key, $_POST[$key]);
                $validation = static::validate($str, $key, $value, $_POST);

                if ($validation['status'] == 'error'):
                    $canSubmit = false;
                endif;

                if ( in_array($key, static::DB_NAMES[$str])):
                    $data[$key] = $value;
                endif;

                $_SESSION[$str . '--' . $key . '-value'] = $value;
                $_SESSION[$str . '--' . $key . '-status'] = $validation['status'];
                $_SESSION[$str . '--' . $key . '-message'] = $validation['message'];
            }

            if ($canSubmit && !static::isCaptchaValid($_POST, $str)) {
                $_SESSION[$str . '--captcha-status'] = 'error';
                wp_redirect( $referer );
                exit;
            }

            unset($_SESSION['employees--step']);

            if ($canSubmit):

                $data['referer'] = ($_SESSION['referer'] !== null) ? $_SESSION['referer'] : null;
                $data['request'] = ($_SESSION['request'] !== null) ? $_SESSION['request'] : null;
                $insert_id = static::insert_row_to_db($data, array( '%s', '%s', '%s', '%s', '%s', '%s' ));

                $_SESSION[$str . '--step'] = 2;
                $_SESSION[$str . '--id'] = $insert_id;
                $_SESSION[$str . '--s'] = 1;
            endif;

        elseif ($_POST['s'] == "2" && $_SESSION['employees--id']):

            if (!static::isCaptchaValid($_POST, $str)) {
                $_SESSION[$str . '--captcha-status'] = 'error';
                wp_redirect( $referer );
                exit;
            }

            $data = [
                'city' => static::getValue('city', $_POST['city']),
                'city_id' => static::getValue('city_id', $_POST['city_id']),
                'message' => static::getValue('message', $_POST['message']),
                'confirm' => static::getValue('confirm', $_POST['confirm']),
                'updated' => current_time('Y-m-d H:i:s')
            ];

            add_filter( 'upload_dir', array( $this, 'change_upload_dir' ) );
            add_filter( 'sanitize_file_name', array( $this, 'change_upload_filename' ), 10 );

            $allowed_file_types = array('application/pdf', 'application/doc', 'application/docx', 'application/rtf', 'application/txt', 'application/odf', 'application/msword', 'image/jpg', 'image/jpeg', 'image/png');

            # upload file
            if($_FILES && $_FILES['cv'] && $_FILES['cv']['error'] == 0):
                $arr_file_type = wp_check_filetype( basename( $_FILES['cv']['name'] ) );

                if (!in_array( $arr_file_type['type'], $allowed_file_types ) ) {
                    $_SESSION['employees--cv-status'] = 'error';
                    $_SESSION['employees--cv-message'] = 'Incorrect file';
                    $_SESSION['employees--cv-translation'] = __('Incorrect file', 'biuro-contacts');
                    wp_redirect( $referer );
                    exit;
                }

                # gali nusirody: 'mimes' => array('csv' => 'text/csv')
                $movefile = wp_handle_upload($_FILES['cv'], [
                    'test_form' => false,
                    //'unique_filename_callback' => 'custom_file_name'
                ]);

                if ( $movefile && ! isset( $movefile['error'] ) ) {
                    $data['cv'] = static::getValue('cv', $movefile['url']);
                } else {
                    $_SESSION['employees--cv-status'] = 'error';
                    $_SESSION['employees--cv-message'] = $movefile['error'];
                    wp_redirect( $referer );
                    exit;
                }

            endif;

            remove_filter( 'sanitize_file_name', array( $this, 'change_upload_filename' ), 10 );
            remove_filter( 'upload_dir', array( $this, 'change_upload_dir' ) );

            unset($_SESSION['employees--step']);

            $data['referer'] = ($_SESSION['referer'] !== null) ? $_SESSION['referer'] : null;
            $data['request'] = ($_SESSION['request'] !== null) ? $_SESSION['request'] : null;

            static::update_row_in_db($data, $_SESSION['employees--id'], ['%s', '%d', '%s', '%d', '%s', '%s', '%s', '%s' ]);

            $_SESSION['employees--s'] = 2;
        endif;

        // $hash = ($_SESSION['employees--name-status'] != 'error' && $_SESSION['employees--phone-status'] != 'error' &&  $_SESSION['employees--agree-status'] == 'error') ? '#agree' : '#action';
        $hash = '#action';

        if ($_POST['s'] == "1"):
            wp_redirect( $referer . $hash );
        else:
            wp_redirect( $referer );
        endif;
        exit;
    }

/* --------------------------- employers ---------------------------

    /**
     * Processes shortcode biuro-contacts--employers
     *
     * @param   array   $attr       The attributes from the shortcode
     *
     * @return  mixed   $output     Output of the buffer
     */
    public function employers_form( $attr = array() ) {
        ob_start();

        include_once('partials/biuro-contacts-public--employers.php');

        $output = ob_get_contents();

        ob_end_clean();

        static::delete_sessions('employers');

        return $output;
    } // employers_form()

    public function employers_post() {

        $nonce = $_POST['_wpnonce'];
        $referer = $_POST['_wp_http_referer'];

        if ( !isset( $nonce ) || !wp_verify_nonce($nonce, 'employers_post_nonce' )) {
            wp_redirect( $referer );
            exit;
        }

        if ( $referer && substr($referer, -1) == '?') {
            $referer = substr($referer, 0, -1);
        }

        unset($_SESSION['employers--step']);

        $str = 'employers';

        if ($_POST['s'] == "1"):
            $canSubmit = true;

            $data = [
                'created' => current_time('Y-m-d H:i:s')
            ];

            foreach (static::FIELDS[$str] as $key) {
                $value = static::getValue($key,  $_POST[$key]);
                $validation = static::validate($str, $key, $value,  $_POST);

                if ($validation['status'] == 'error'):
                    $canSubmit = false;
                endif;

                if ( in_array($key, static::DB_NAMES[$str])):
                    $data[$key] = $value;
                endif;

                $_SESSION[$str . '--' . $key . '-value'] = $value;
                $_SESSION[$str . '--' . $key . '-status'] = $validation['status'];
                $_SESSION[$str . '--' . $key . '-message'] = $validation['message'];
            }

            if ($canSubmit && !static::isCaptchaValid($_POST, $str)) {
                $_SESSION[$str . '--captcha-status'] = 'error';
                wp_redirect( $referer );
                exit;
            }

            if ($canSubmit):
                $data['referer'] = ($_SESSION['referer'] !== null) ? $_SESSION['referer'] : null;
                $data['lead_id'] = null;
                $data['request'] = ($_SESSION['request'] !== null) ? $_SESSION['request'] : null;

                $insert_id = static::insert_row_to_db($data, array( '%s', '%s', '%s', '%s', '%s', '%d', '%s' ), 'biuro_employers');

                if (!in_array(strstr($_SERVER['SERVER_NAME'], '.biuro', true), array('dev'))):
                    static::sendEmail( static::emailContent($data['name'], static::BIURO_INSIGHTLY[get_current_blog_id()], $data['phone'], $data['email']), $data['name']);

                    $return = static::sendPost($data['name'], static::BIURO_INSIGHTLY[get_current_blog_id()], $data['phone'], $data['email'], $data['referer'], $data['request']);

                    if ($return['LEAD_ID']):
                        static::update_row_in_db(['lead_id' => $return['LEAD_ID']], $insert_id, ['%d'] , 'biuro_employers');
                    endif;
                endif;

                $_SESSION[$str . '--step'] = 2;
                $_SESSION[$str . '--id'] = $insert_id;
                $_SESSION['employers--s'] = 1;
            endif;

        elseif ($_POST['s'] == "2"):

            if (!static::isCaptchaValid($_POST, $str)) {
                $_SESSION[$str . '--captcha-status'] = 'error';
                wp_redirect( $referer );
                exit;
            }

            $data = [
                'company' => static::getValue('city', $_POST['company']),
                'message' => static::getValue('message', $_POST['message']),
                'confirm' => static::getValue('confirm', $_POST['confirm']),
                'updated' => current_time('Y-m-d H:i:s')
            ];

            static::update_row_in_db($data, $_SESSION['employers--id'], ['%s', '%s', '%d', '%s'] , 'biuro_employers');

            $employerData = static::fetchRow($_SESSION['employers--id'], 'biuro_employers');

            if (!in_array(strstr($_SERVER['SERVER_NAME'], '.biuro', true), array('dev'))):
                static::sendEmail( static::emailContent($employerData['name'], static::BIURO_INSIGHTLY[get_current_blog_id()], $employerData['phone'], $employerData['email'], $data['company'], $data['message']), $employerData['name'] );

                if ($employerData['lead_id']):
                    $return = static::curlPut($employerData['lead_id'], $employerData['name'], static::BIURO_INSIGHTLY[get_current_blog_id()], $employerData['phone'], $employerData['email'], $data['company'], $data['message'], $employerData['referer'], $employerData['request']);
                endif;
            endif;

            $_SESSION['employers--s'] = 2;

            # isvalom klaidos teksta;
            unset($_SESSION[$str.'file_error']);
        endif;


        // $hash = ($_SESSION['employers--name-status'] != 'error' && $_SESSION['employers--phone-status'] != 'error' &&  $_SESSION['employers--agree-status'] == 'error') ? '#agree' : '#action';
        $hash = '#action';

        if ($_POST['s'] == "1"):
            wp_redirect( $referer . $hash );
        else:
            wp_redirect( $referer );
        endif;
        exit;
    }

    public static function curlPut($leadId, $firstName, $lastName, $phone, $email, $organization, $comment, $referer = '', $landingPage = '') {
            $service_url = 'https://api.insight.ly/v2.2/Leads';
            $ch = curl_init($service_url);

            $firstName =$firstName;
            $lastName = $lastName;
            $organization = $organization;
            $comment = $comment;

            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json',
                'Authorization: Basic ' . base64_encode('db7f06b9-1d5d-433b-8811-bf32fc614c79')
            ));

            curl_setopt($ch, CURLOPT_HEADER, 0);
            curl_setopt($ch, CURLOPT_TIMEOUT, 30);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
            curl_setopt($ch, CURLOPT_VERBOSE, 1);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, true);


            $data = '{
                "LEAD_ID": "'.$leadId.'",
                "FIRST_NAME": "'.$firstName.'",
                "LAST_NAME": "'.$lastName.'",
                "organization_name": "'.$organization.'",
                "mobile_phone_number": "'.$phone.'",
                "email_address": "'.$email.'",
                "Service_Country_L__c": "'.$lastName.'",
                "CUSTOMFIELDS": [ {
                    "CUSTOM_FIELD_ID": "Service_Country_L__c",
                    "FIELD_VALUE": "'.$lastName.'"
                    },
                    {
                    "CUSTOM_FIELD_ID": "Comments_L__c",
                    "FIELD_VALUE": "'.$comment.'"
                    },
                    {
                    "CUSTOM_FIELD_ID": "REFERER_L__c",
                    "FIELD_VALUE": "'.$referer.'"
                    },
                    {
                    "CUSTOM_FIELD_ID": "LANDING_PAGE_L__c",
                    "FIELD_VALUE": "'.$landingPage.'"
                    }
                ]
            }';

            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
            $return = curl_exec($ch);
            $err = curl_errno($ch);
            $msg = curl_error($ch);

            curl_close($ch);

            return json_decode($return, true);
    }

    public static function sendPost($firstName, $lastName, $phone, $email, $referer = '', $landingPage = '') {
            $service_url = 'https://api.insight.ly/v2.2/Leads';
            $ch = curl_init($service_url);

            $firstName = $firstName;
            $lastName = $lastName;

            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json',
                'Authorization: Basic ' . base64_encode('db7f06b9-1d5d-433b-8811-bf32fc614c79')
            ));

            curl_setopt($ch, CURLOPT_HEADER, 0);
            curl_setopt($ch, CURLOPT_TIMEOUT, 30);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
            curl_setopt($ch, CURLOPT_VERBOSE, 1);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, true);

            $data = '{
                "FIRST_NAME": "'.$firstName.'",
                "LAST_NAME": "'.$lastName.'",
                "mobile_phone_number": "'.$phone.'",
                "email_address": "'.$email.'",
                "Service_Country_L__c": "'.$lastName.'",
                "CUSTOMFIELDS": [ {
                    "CUSTOM_FIELD_ID": "Service_Country_L__c",
                    "FIELD_VALUE": "'.$lastName.'"
                    },
                    {
                    "CUSTOM_FIELD_ID": "REFERER_L__c",
                    "FIELD_VALUE": "'.$referer.'"
                    },
                    {
                    "CUSTOM_FIELD_ID": "LANDING_PAGE_L__c",
                    "FIELD_VALUE": "'.$landingPage.'"
                    }
                ]
            }';
            curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
            $return = curl_exec($ch);
            $err = curl_errno($ch);
            $msg = curl_error($ch);

            curl_close($ch);

            return json_decode($return, true);
    }

    public static function sendBlockedEmail($message, $section) {
        $to = ($section == 'employers') ? 'pardavimai@biuro.lt' : 'vilniuscv@biuro.lt';

        $subject = 'Biuro blocked contact form';

        $headers = "From: Biuro <no-reply@biuro.lt>" . "\r\n";

        try {
            wp_mail( $to, $subject, $message, $headers );
        } catch (Exception $e) {
            debug($e);
        }
    }

    public static function sendEmail($message, $name) {
        $to = 'pardavimai@biuro.lt';

        $subject = 'Biuro employers contact form';

        $headers = "From: Biuro <no-reply@biuro.lt>" . "\r\n";

        try {
            wp_mail( $to, $subject, $message, $headers );
        } catch (Exception $e) {
            debug($e);
        }
    }

    public static function emailBlockedContent($post, $captcha, $response = array()) {
        $message = '';

        $message .= 'Date: ' . date('Y-m-d H:i:s') . "\r\n";
        $message .= 'Website: ' . $_SERVER['SERVER_NAME'] . "\r\n";
        $message .= 'Location: ' . $_SERVER['HTTP_REFERER'] . "\r\n" . "\r\n";

        $message .= 'Name: ' . $post['name'] . "\r\n";
        $message .= 'Phone: ' . $post['phone'] . "\r\n";
        $message .= 'Email: ' . $post['email'] . "\r\n";

        if ($post['city']):
            $message .= 'City: ' . $post['city'] . "\r\n";
        endif;

        if ($post['company']):
            $message .= 'Company: ' . $post['company'] . "\r\n";
        endif;

        if ($post['message']):
            $message .= 'Message: ' . $post['message'] . "\r\n";
        endif;

        $message .= "\r\n";

        if ($captcha):
            if (isset($response['error-codes']) && is_array($response['error-codes']) && !empty($response['error-codes'])):
                $message .= 'Error codes:';

                foreach ($response['error-codes'] as &$code) :
                    $message .= ' ' . $code;
                endforeach;

                $message .= "\r\n";
            endif;
        else:
            $message .= 'Reason: No recaptcha code' . "\r\n";
        endif;

        return $message;
    }

    public static function emailContent($firstName, $lastName, $phone, $email, $organization = null, $comment = null) {
        $message = '';

        $message .= 'Firstname: ' . $firstName . "\r\n";
        $message .= 'Lastname: ' . $lastName . "\r\n";
        $message .= 'Phone no.: ' . $phone . "\r\n";
        $message .= 'Email: ' . $email . "\r\n";

        if ($organization || $comment):
            $message .= 'Company: ' . $organization . "\r\n";
            $message .= 'Message: ' . $comment . "\r\n";
        endif;

        return $message;
    }
/* --------------------------- position ---------------------------

    /**
     * Processes shortcode biuro-contacts--position
     *
     * @param   array   $attr       The attributes from the shortcode
     *
     * @return  mixed   $output     Output of the buffer
     */
    public function position_form( $attr = array() ) {
        ob_start();

        if ($attr['job_id']):
            $job_id = $attr['job_id'];
        endif;

        if ($attr['step']):
            $step = $attr['step'];
        endif;

        include_once('partials/biuro-contacts-public--position.php');

        unset($_SESSION['position--step']);

        $output = ob_get_contents();

        ob_end_clean();

        static::delete_sessions('position');

        return $output;
    } // position_form()

    public function position_post() {

        $nonce = $_POST['_wpnonce'];
        $referer = $_POST['_wp_http_referer'];


        if ( !isset( $nonce ) || !wp_verify_nonce($nonce, 'position_post_nonce' )) {
            wp_redirect( $referer );
            exit;
        }

        if ( $referer && substr($referer, -1) == '?') {
            $referer = substr($referer, 0, -1);
        }

        // unset($_SESSION['position--step']);

        if ($_POST['s'] == "1"):

            $_SESSION['position--step'] = 2;
            $_SESSION['position--s'] = 1;

            // $hash = '#agree';
            $hash = '#action';

        elseif ($_POST['s'] == "2"):

            $_SESSION['position--step'] = 2;

            $canSubmit = true;

            $str = 'position';

            $data = [
                'created' => current_time('Y-m-d H:i:s'),
                'updated' => current_time('Y-m-d H:i:s')
            ];

            foreach (static::FIELDS[$str] as $key) {
                $value = static::getValue($key, $_POST[$key]);
                $validation = static::validate($str, $key, $value, $_POST);

                if ($validation['status'] == 'error'):
                    $canSubmit = false;
                endif;

                if ( in_array($key, static::DB_NAMES[$str])):
                    $data[$key] = $value;
                endif;

                $_SESSION[$str . '--' . $key . '-value'] = $value;
                $_SESSION[$str . '--' . $key . '-status'] = $validation['status'];
                $_SESSION[$str . '--' . $key . '-message'] = $validation['message'];
            }

            if ($canSubmit && !static::isCaptchaValid($_POST, $str)) {
                $_SESSION[$str . '--captcha-status'] = 'error';
                wp_redirect( $referer );
                exit;
            }

            // $hash = ($_SESSION['position--name-status'] != 'error' && $_SESSION['position--phone-status'] != 'error' && $_SESSION['position--agree-status'] == 'error') ? '#agree' : '#action';
            $hash = '#action';

            if ($canSubmit):

                add_filter( 'upload_dir', array( $this, 'change_upload_dir' ) );
                add_filter( 'sanitize_file_name', array( $this, 'change_upload_filename' ), 10 );

                $allowed_file_types = array('application/pdf', 'application/doc', 'application/docx', 'application/rtf', 'application/txt', 'application/odf', 'application/msword', 'image/jpg', 'image/jpeg', 'image/png');

                # upload file
                if($_FILES && $_FILES['cv'] && $_FILES['cv']['error'] == 0):
                    $arr_file_type = wp_check_filetype( basename( $_FILES['cv']['name'] ) );

                    if (!in_array( $arr_file_type['type'], $allowed_file_types ) ) {
                        $_SESSION['position--cv-status'] = 'error';
                        $_SESSION['position--cv-message'] = 'Incorrect file';
                        $_SESSION['position--cv-translation'] = __('Incorrect file', 'biuro-contacts');
                        wp_redirect( $referer );
                        exit;
                    }

                    # gali nusirody: 'mimes' => array('csv' => 'text/csv')
                    $movefile = wp_handle_upload($_FILES['cv'], [
                        'test_form' => false,
                        //'unique_filename_callback' => 'custom_file_name'
                    ]);

                    if ( $movefile && ! isset( $movefile['error'] ) ) {
                        $data['cv'] = static::getValue('cv', $movefile['url']);
                    } else {
                        $_SESSION['position--cv-status'] = 'error';
                        $_SESSION['position--cv-message'] = $movefile['error'];
                        wp_redirect( $referer );
                        exit;
                    }

                endif;

                remove_filter( 'sanitize_file_name', array( $this, 'change_upload_filename' ), 10 );
                remove_filter( 'upload_dir', array( $this, 'change_upload_dir' ) );

                unset($_SESSION['position--step']);

                $data['referer'] = ($_SESSION['referer'] !== null) ? $_SESSION['referer'] : null;
                $data['request'] = ($_SESSION['request'] !== null) ? $_SESSION['request'] : null;

                $insert_id = static::insert_row_to_db($data, array( '%s', '%s', '%s', '%s', '%s', '%s', '%d', '%s', '%d', '%d', '%s', '%s', '%s' ));

                $_SESSION['employees--id'] = $insert_id;
                $_SESSION['position--s'] = 2;

                $hash = '';
            endif;

        endif;

        wp_redirect( $referer . $hash );
        exit;
    }

    public function recommend_form( $attr = array() ) {
        ob_start();

        include_once('partials/biuro-contacts-public--recommend.php');

        $output = ob_get_contents();

        ob_end_clean();

        static::delete_sessions('recommend');

        return $output;
    } // recommend_form()

    public function recommend_post() {

        $nonce = $_POST['_wpnonce'];
        $referer = $_POST['_wp_http_referer'];

        if ( !isset( $nonce ) || !wp_verify_nonce($nonce, 'recommend_post_nonce' )) {
            wp_redirect( $referer );
            exit;
        }

        if ( $referer && substr($referer, -1) == '?') {
            $referer = substr($referer, 0, -1);
        }

        $canSubmit = true;

        $str = 'recommend';

        if ($_POST['s'] == "1"):
            $data = [
                'created' => current_time('Y-m-d H:i:s'),
            ];

            foreach (static::FIELDS[$str] as $key) {
                $value = static::getValue($key, $_POST[$key]);
                $validation = static::validate($str, $key, $value, $_POST);

                if ($validation['status'] == 'error'):
                    $canSubmit = false;
                endif;

                if ( in_array($key, static::DB_NAMES[$str])):
                    $data[$key] = $value;
                endif;

                $_SESSION[$str . '--' . $key . '-value'] = $value;
                $_SESSION[$str . '--' . $key . '-status'] = $validation['status'];
                $_SESSION[$str . '--' . $key . '-message'] = $validation['message'];
            }

            if ($canSubmit && !static::isCaptchaValid($_POST, $str)) {
                $_SESSION[$str . '--captcha-status'] = 'error';
                wp_redirect( $referer );
                exit;
            }

            $hash = '#action';

            if ($canSubmit):
                $data['referer'] = ($_SESSION['referer'] !== null) ? $_SESSION['referer'] : null;
                $data['request'] = ($_SESSION['request'] !== null) ? $_SESSION['request'] : null;

                $insert_id = static::insert_row_to_db($data, array( '%s', '%s', '%s', '%s', '%s', '%d', '%s', '%s', '%s', '%s', '%d', '%s', '%s' ), 'biuro_recommend');

                $_SESSION[$str . '--id'] = $insert_id;
                $_SESSION[$str . '--s'] = 1;

                static::delete_sessions('recommend');
            endif;
        endif;

        wp_redirect( $referer . $hash );
        exit;
    }



    /**
    * @method getContacts
    * get contacts data from DB for API endpoint "contacts" endpoint;
    * @param WP_REST_Request $request
    * @return json;
    */
    public static function getContacts(WP_REST_Request $request) {
        global $wpdb;
        $params = $request->get_params();

        if ( !$params ) {
            exit;
        }

        $from = (string) $params['from'];
        $orderBy = (string) $params['by'];

        if(!$from || !in_array($orderBy, ['created', 'updated'])) {
            exit;
        }

        $sql = "SELECT * FROM `" . $wpdb->prefix . "biuro_employees`
            where ". $orderBy. " >= '". $from. "' ORDER BY ". $orderBy ." DESC LIMIT 50";
        return $wpdb->get_results($sql, ARRAY_A);
    }

    /**
    * @method getRecommended
    * get contacts data from DB for API endpoint "recommended" endpoint;
    * @param WP_REST_Request $request
    * @return json;
    */
    public static function getRecommended(WP_REST_Request $request) {
        global $wpdb;
        $params = $request->get_params();

        if ( !$params ) {
            exit;
        }

        $from = (string) $params['from'];
        $orderBy = (string) $params['by'];

        if(!$from || !in_array($orderBy, ['created', 'updated'])) {
            exit;
        }

        $sql = "SELECT * FROM `" . $wpdb->prefix . "biuro_recommend`
            where ". $orderBy. " >= '". $from. "' ORDER BY ". $orderBy ." DESC LIMIT 50";
        return $wpdb->get_results($sql, ARRAY_A);
    }

    public static function fetchRow($primaryKey, $table = 'biuro_employees') {
        global $wpdb;

        if(!is_numeric($primaryKey)) {
            exit;
        }

         $sql = "SELECT * FROM `" . $wpdb->prefix. $table. "`
            where id = " . $primaryKey . " LIMIT 1";

        return $wpdb->get_row($sql, ARRAY_A);
    }
}
